import { Component, OnInit, NgModule, Inject, Input } from '@angular/core';
import { MatTableDataSource, MatDialog, MatDialogRef, MAT_DIALOG_DATA, MatDialogModule, MatDialogConfig } from '@angular/material';

import { Location } from '@angular/common';
// import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { Observable, Subject } from 'rxjs';
import { Subscription, of } from 'rxjs';
import { fromEvent } from 'rxjs';
import { timer } from 'rxjs/observable/timer';
import { repeat, takeUntil, skip, switchMap, take, filter } from 'rxjs/operators';
import { Router, ActivatedRoute, ParamMap } from '@angular/router';

import {Strudelapp} from '../strudelapp';
import { StrudelappsService } from '../strudelapps.service';
import { TesService } from '../tes.service';
import {BackendSelectionService } from '../backend-selection.service';
import { AuthorisationService } from '../authorisation.service';
import { Identity } from '../identity';
import { Computesite, Health } from '../computesite';
import { LogoutdialogComponent } from '../logoutdialog/logoutdialog.component';

import { SshAuthzServer } from '../identity';
import { ComputesitesService } from '../computesites.service';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { SettingsService} from '../settings.service';




export interface Quota {
  resource: string;
  usage: string;
  quota: string;
}




@Component({
  selector: 'app-launcher',
  templateUrl: './launcher.component.html',
  styleUrls: ['./launcher.component.css']
})
export class LauncherComponent implements OnInit {
  public strudelapps: Strudelapp[];

  public appSubject: BehaviorSubject<Strudelapp>;
  public authorised: boolean;
  public identity: Identity;
  public identitySubject: BehaviorSubject<Identity>;

  public identities: Identity[];
  public sshauthzservers: SshAuthzServer[];
  private launchwindow: any;
  private launchwindowWatcher: any;
  private subscriptions: Subscription[];
  //public quotas: BehaviorSubject<any[]>;
  displayedColumns: string[] = [];
  private identity$: Observable<Identity>;
  private app$: Observable<Strudelapp>;
  private app: Strudelapp;


  constructor( public dialog: MatDialog,
                public tesService: TesService,
                public backendSelectionService: BackendSelectionService,
                public authService: AuthorisationService,
                public computeSitesService: ComputesitesService,
                private router: Router,
                private route: ActivatedRoute,
                private strudelappsService: StrudelappsService,
                private settingsService: SettingsService,
                ) {
      this.subscriptions = [];
      this.subscriptions.push(this.authService.sshAuthzServers.subscribe(o => {this.updateSshAuthZServers(o)}));
      this.subscriptions.push(this.authService.loggedInAuthZ.subscribe(o => this.navLogin(o)));
      this.subscriptions.push(this.computeSitesService.appidentities.subscribe(o => this.getHealth(o)));
      this.appSubject = new BehaviorSubject<Strudelapp>(null);
      this.identitySubject = new BehaviorSubject<Identity>(null);
      this.identity = null;
    //this.quotas = new BehaviorSubject<any[]>([]);
  }

  countErrors(a: Health[], b: Health[]) {
    var count: number = 0
    var h: Health;
    if (a != null) {
      for (h of a) {
        if (h.stat == 'error' || h.stat == 'warn') {
          count++;
        }
        if(!isNaN(parseInt(h.stat))) {
          count = count + parseInt(h.stat);
        }
      }
    }
    if (b != null) {
      for (h of b) {
        if (h.stat == 'error' || h.stat == 'warn') {
          count++;
        }
        if(!isNaN(parseInt(h.stat))) {
          count = count + parseInt(h.stat);
        }
      }
    }
    return count;
  }

  calculateCols(data) {
    return data.map(function(r) { return r.key });
  }

  humanKBytes(n: number) {
    if (n > 1024*1024*1024) {
      let v = n/1024/1024/1024;
      return v.toFixed(0)+'TB';
    }
    if (n > 1024*1024) {
      let v = n/1024/1024;
      return v.toFixed(0)+'GB';
    }
    if (n > 1024) {
      let v = n/1024;
      return v.toFixed(0)+'MB';
    }
    return '0 MB';
  }

  rowClass(row) {
    if (row.stat == 'error' || row.stat == 'warn') {
      return 'health-warn';
    } else {
      return 'health-ok';
    }
  }

  navLogin(o) {
    if (o == null) {
      return
    }
    if (o.length == 0) {
      this.router.navigate(['/login']);
    } else {
    }
  }

  getHealth(o) {
    if (o == null) {
      return
    }
    for (let id of o) {
      this.tesService.getHealthAlerts(id);
    }
  }

  updateSshAuthZServers(o) {
    this.sshauthzservers = o;
  }

  ngOnInit() {
    this.strudelapps = [];
    /*this.identity$ = this.route.paramMap.pipe(
      switchMap((params: ParamMap) => this.getId(params.get('site')))
    );*/
    this.route.paramMap.subscribe((params: ParamMap) => this.updateIdApp(params));
    /*this.app$ = this.route.paramMap.pipe(
      switchMap((params: ParamMap) => this.getApp(params.get('app')))
    );
    this.identity$.subscribe((v) => {console.log('id changed'); console.log(v)});
    this.identity$.subscribe((v) => this.identity = v);
    this.app$.subscribe((v) => { console.log('app changed'); console.log(v)});*/
    //this.site$.subscribe ( (v) => { this.getId(v) });
    //this.app$.subscribe( (v) => { this.getApp(v) });
  }

  updateIdApp(params: ParamMap) {
    var ids: string = params.get('site');
    var apps: string = params.get('app');
    if (ids === null) {
      this.identitySubject.next(null);
      return;
    }
    var id: Identity = this.getId(ids);
    if (id === null) {
      console.log('id is null, callback updateIdApp');
      this.computeSitesService.appidentities.pipe(skip(1),filter((v) => v !== null && v.length > 0),take(1)).subscribe(() => this.updateIdApp(params));
      return
    }
    if (id !== this.identitySubject.value) {
      this.identitySubject.next(id);
    }     

    if (apps === null) {
      apps = 'acocuntinfo';
      //this.appSubject.next(null);
      //return;
    }
    if (id.site.appCatalog.value === undefined || id.site.appCatalog.value === null || (id.site.appCatalog.value).length == 0) {
      id.site.appCatalog.pipe(filter((v) => v !== null && v.length > 0)).subscribe(() => this.updateIdApp(params));
      return
    }
    var app: Strudelapp = this.getApp(id,apps);
    if (app !== this.appSubject.value) {
      this.appSubject.next(app);
    }}

  getApp(id,v: any): Strudelapp {
    if (id == 'accountinfo') {
      return null;
    }
    return this.strudelappsService.getApp(v,id.site.appCatalog.value);
  }

  getId(v: any) {
    if (v === null) {
      console.error('getId called with null value');
      return null;
    }
    if (this.computeSitesService.appidentities.value === null || this.computeSitesService.appidentities.value == []) {
      console.error('no appidentities yet? call back latter');
      this.computeSitesService.appidentities.pipe(filter((v) => v !== null)).subscribe(() => this.getId(v));
      return null
    }

    for ( let id of this.computeSitesService.appidentities.value) {
      if (v == id.site.name) {
        return id
      }
    }
    console.error('didnt find a matching id');
    console.log(v);
    console.log(this.computeSitesService.appidentities.value);
    return null;
  }


  ngOnDestroy() {
    for (let s of this.subscriptions) {
      s.unsubscribe();
    }
  }

  selectId(id: Identity) {
    this.identitySubject.next(id);
    this.identity = id;
    this.appSubject.next(null);

    //if (id != null) {
      // We will subscribe to account alerts until the id changes (in which case we will subscribe to account alerts
      // on the next id
      // BUT
      // since identitySubject is a behaviorsubject and already has a value, we must skip(1)
    //  id.accountalerts.pipe(takeUntil(this.identitySubject.pipe(skip(1)))).subscribe((msgs) => { this.quotas.next(msgs.filter(q => q.type =='quota')) });
    //}
  }

  selectApp(app: Strudelapp) {
    this.appSubject.next(app);
  }


}
